package com.oracle.coherence.swingdemo;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.geom.Ellipse2D;
import java.util.Comparator;
import java.util.LinkedList;
import java.util.Map;
import java.util.Set;
import java.util.TreeSet;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;

import javax.swing.JFrame;

import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.axis.CategoryAxis;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.labels.ItemLabelAnchor;
import org.jfree.chart.labels.ItemLabelPosition;
import org.jfree.chart.labels.StandardCategoryItemLabelGenerator;
import org.jfree.chart.plot.CategoryMarker;
import org.jfree.chart.plot.CategoryPlot;
import org.jfree.chart.plot.CombinedDomainCategoryPlot;
import org.jfree.chart.plot.PiePlot3D;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.renderer.category.BarRenderer3D;
import org.jfree.chart.renderer.category.LineAndShapeRenderer;
import org.jfree.data.category.CategoryDataset;
import org.jfree.data.category.DefaultCategoryDataset;
import org.jfree.data.general.DefaultPieDataset;
import org.jfree.ui.Layer;
import org.jfree.ui.TextAnchor;
import org.jfree.util.Rotation;

import com.oracle.coherence.swingdemo.agents.DataDistributionQueryInvocable;
import com.oracle.coherence.util.tuples.Pair;
import com.oracle.coherence.util.tuples.Tuple4;
import com.oracle.coherence.util.tuples.TupleN;
import com.tangosol.net.CacheFactory;
import com.tangosol.net.DistributedCacheService;
import com.tangosol.net.InvocationService;
import com.tangosol.net.Member;
import com.tangosol.net.NamedCache;
import com.tangosol.util.Filter;
import com.tangosol.util.aggregator.Count;
import com.tangosol.util.aggregator.DistinctValues;
import com.tangosol.util.aggregator.DoubleSum;
import com.tangosol.util.filter.EqualsFilter;

/**
 * <p>The entry point for the swing demo.</p>
 *
 * @author Brian Oliver (brian.oliver@oracle.com)
 */
@SuppressWarnings("serial")
public class GUI extends JFrame {

	private static String VERSION = "2.0";
	
    //the scheduler for background tasks (like determining the data grid distribution)
    private ScheduledExecutorService scheduler;
	
    //the order distribution data set
    private DefaultCategoryDataset orderDistributionDataset;
    
    //the order symbols pie data set
    private DefaultPieDataset positionSymbolsDataset;
    
    //the time for aggregation
    private DefaultCategoryDataset aggregationTimeDataset;
    private LinkedList<Tuple4<Integer, Integer, Long, Boolean>> aggregationTimes;  //Tuple4<cache size, nr aggregations, total time (ms), did membership change?>
    private int nrTasksExecuted;
    private long totalAggregationTime;
    private long minAggregationTime;
    private long maxAggregationTime;

    //the throughput for each query (orders per second)
    private DefaultCategoryDataset aggregationThroughputDataset;
    
    //the current valuation
    private double valuation;
    
    //the list of member information (per query)
    private LinkedList<TreeSet<Pair<Member, TupleN>>> memberInformation; //results from the DataDistributionQueryInvocable
    
    //the memory consumption dataset
    private DefaultCategoryDataset memoryConsumptionDataset;
    
    
    /**
     * Creates new form Main
     */
    public GUI() {    	
    	//create the datasets for our graphs
    	orderDistributionDataset = new DefaultCategoryDataset();
    	positionSymbolsDataset = new DefaultPieDataset();
    	aggregationTimeDataset = new DefaultCategoryDataset();
    	aggregationThroughputDataset = new DefaultCategoryDataset();
    	memoryConsumptionDataset = new DefaultCategoryDataset();
    	
    	//the latency information for past 'N' aggregations
    	aggregationTimes = new LinkedList<Tuple4<Integer, Integer, Long, Boolean>>();
    	totalAggregationTime = 0;
    	nrTasksExecuted = 0;
    	minAggregationTime = 0;
    	maxAggregationTime = 0;
    	
    	//the current valuation
    	valuation = 0.0;
    	
    	//the member information
    	memberInformation = new LinkedList<TreeSet<Pair<Member,TupleN>>>();   	
    	
    	//initialise the GUI components
        initComponents();
        
        CacheFactory.ensureCluster();
        
        //the scheduler for background queries etc (to update the GUI asynchronously)
        scheduler = Executors.newScheduledThreadPool(1);
        
        //schedule a task to determine the status of each member for a cache
        scheduler.scheduleWithFixedDelay(new Runnable() {
        	
        	private int nrStorageEnabledMembersForLastExecution = 0;
        	
			@SuppressWarnings("unchecked")
			public void run() {
				try {
	                NamedCache namedCache = CacheFactory.getCache("positions");
	                InvocationService invocationService = (InvocationService)CacheFactory.getService("InvocationService");
	                
	                //determine the storage enabled members for the membership query
	                Set<Member> storageEnabledMembers = ((DistributedCacheService)namedCache.getCacheService()).getStorageEnabledMembers();
	                
	                //determine the member information
					Map<Member, TupleN> results = (Map<Member, TupleN>)invocationService.query(new DataDistributionQueryInvocable(namedCache.getCacheName()), storageEnabledMembers);				
					
					//create a treeset of TupleN sorted by the memberid
					TreeSet<Pair<Member, TupleN>> memberResults = new TreeSet<Pair<Member, TupleN>>(new Comparator<Pair<Member, TupleN>>() {
						public int compare(Pair<Member, TupleN> p1, Pair<Member, TupleN> p2) {
							return p1.getX().getId() - p2.getX().getId();
						}
					});				
			        for (Map.Entry<Member, TupleN> entry : results.entrySet()) {
			        	memberResults.add(new Pair(entry));
			        }
			        memberInformation.add(memberResults);
			        
					//determine the number of orders so we can use percentages
					int nrOrders = namedCache.size();
					DefaultCategoryDataset workingOrderDistributionDataset = new DefaultCategoryDataset();
					
			        for (Pair<Member, TupleN> pair : memberResults) {
			        	Member member = pair.getX();
			        	TupleN memberDetails = pair.getY();
			        	workingOrderDistributionDataset.addValue((Integer)memberDetails.get(0), String.format("Member %d", memberDetails.get(1)), member.getAddress().getHostName() + "");
			        }
			        orderDistributionDataset = workingOrderDistributionDataset;
			        
			        //determine the number of each type of order
			        int nrAggregations = 0;
			        long timeStart = System.currentTimeMillis();
			    	DefaultPieDataset workingPositionSymbolsDataset = new DefaultPieDataset();
			        TreeSet<String> symbols = new TreeSet<String>();
			        symbols.addAll((Set<String>)namedCache.aggregate((Filter)null, new DistinctValues("getSymbol")));
			        nrAggregations++;
	                for (String symbol : symbols) {
	                	int count = (Integer)namedCache.aggregate(new EqualsFilter("getSymbol", symbol), new Count());
	                	nrAggregations++;
	                	workingPositionSymbolsDataset.setValue(String.format("%s (%,d=%.1f%%)", symbol, count, count / (double)nrOrders * 100.0), count);
	                }
	                positionSymbolsDataset = workingPositionSymbolsDataset;
	                
	                //determine the value for the positions
	                valuation = (Double)namedCache.aggregate((Filter)null, new DoubleSum("getValue"));
	                nrAggregations++;
	                
	                long timeEnd = System.currentTimeMillis();
	                long timeElapsed = timeEnd - timeStart;
	                
	                totalAggregationTime += timeElapsed;
	                nrTasksExecuted++;
	                
	                if (nrTasksExecuted == 1) {
	                	minAggregationTime = timeElapsed;
	                	maxAggregationTime = timeElapsed;
	                } else {
	                	if (timeElapsed < minAggregationTime)
	                		minAggregationTime = timeElapsed;
	                	if (timeElapsed > maxAggregationTime)
	                		maxAggregationTime = timeElapsed;
	                }
	                
	                aggregationTimes.addLast(new Tuple4<Integer, Integer, Long, Boolean>(nrOrders, nrAggregations, timeElapsed, nrStorageEnabledMembersForLastExecution != storageEnabledMembers.size()));
	                if (aggregationTimes.size() > 50) { //nr queries to keep
	                	aggregationTimes.removeFirst();
	                	memberInformation.removeFirst();
	                }
	                
	                //construct the query time, throughput datasets
	                DefaultCategoryDataset workingAggregationTimeDataSet = new DefaultCategoryDataset();
	                DefaultCategoryDataset workingAggregationThroughputDataSet = new DefaultCategoryDataset();
	                
	                int queryNr = nrTasksExecuted - aggregationTimes.size() + 1;
	                for (Tuple4<Integer, Integer, Long, Boolean> latency : aggregationTimes) {
	                	String query = String.format("%d", queryNr++);
	                	workingAggregationTimeDataSet.addValue(latency.getC() / (double)latency.getB(), "Average Aggregation Time (ms)", query);
	                	workingAggregationThroughputDataSet.addValue((double)latency.getA() / (double)latency.getC() * latency.getB() * 1000, "Average Aggregation Throughput (orders/second)", query);
	                }
	                aggregationTimeDataset = workingAggregationTimeDataSet;
	                aggregationThroughputDataset = workingAggregationThroughputDataSet;
	                
	                //construct the memory consumption dataset
	            	DefaultCategoryDataset workingMemoryConsumptionDataset = new DefaultCategoryDataset();
	                queryNr = nrTasksExecuted - aggregationTimes.size() + 1;
	                for (TreeSet<Pair<Member, TupleN>> memberQuery : memberInformation) {
	                	String query = String.format("%d", queryNr++);
	
	                	for (Pair<Member, TupleN> memberPair : memberQuery) {
	                		long memoryConsumption = (Long)memberPair.getY().get(3) - (Long)memberPair.getY().get(4);
	                		workingMemoryConsumptionDataset.addValue(memoryConsumption / 1024 / 1000, String.format("Member %d", memberPair.getX().getId()), query);
	                	}
	                }
	                memoryConsumptionDataset = workingMemoryConsumptionDataset;
	                
	                //remember the number of storage enabled members so we can determine and annotate membership changes
	                //between executions of the queries
	                nrStorageEnabledMembersForLastExecution = storageEnabledMembers.size();
	                
			        //now render the charts
			        renderPositionPopularityChart();
			        renderPositionDistributionChart();
			        renderQueryStatisticsChart();
			        
				} catch (Exception exception) {
					//SKIP: We don't care if we get an exception - it's usually a "terminated" exception
				}
			}
        }, 1, 3, TimeUnit.SECONDS);
    }

    
    /**
     * <p>Renders the position popularity pie chart that details the types of financial positions
     * and quantities of those said positions.</p>
     */
    private void renderPositionPopularityChart() {
     	if (aggregationTimes.size() > 0) {
 	        JFreeChart positionSymbolsChart = ChartFactory.createPieChart3D(
 	            String.format("%,d Positions. Total Value $%,.2f", aggregationTimes.getLast().getA(), valuation),  				
 	            positionSymbolsDataset,            
 	            false,                   
 	            false,
 	            false
 	            );
 	        PiePlot3D plot = (PiePlot3D)positionSymbolsChart.getPlot();
 	        plot.setStartAngle(270);
 	        plot.setDirection(Rotation.ANTICLOCKWISE);
 	        plot.setForegroundAlpha(0.60f);
 	        plot.setInteriorGap(0.33);
 	   
 	        chartPositionPopularity.setChart(positionSymbolsChart);
     	}
    }
    
    
    /**
     * <p>Renders a bar chart detailing the distribution of position objects across cluster members.</p>
     */
    private void renderPositionDistributionChart() {
        JFreeChart orderDistributionChart = ChartFactory.createBarChart3D(
	        	"Data Distribution (positions/member)",  
	            "Cluster Members",                  	
	            "# Positions (objects)",  		
	            orderDistributionDataset,   	
	            PlotOrientation.VERTICAL,  		
	            false,                      	
	            false,							
	            false							
	        );
	        
        CategoryPlot categoryPlot = (CategoryPlot)orderDistributionChart.getPlot();
        BarRenderer3D barRenderer3D = (BarRenderer3D)categoryPlot.getRenderer();

        barRenderer3D.setBaseItemLabelGenerator(new StandardCategoryItemLabelGenerator() {
        @Override
        	public String generateLabel(CategoryDataset dataset, int series, int category) {
        		return dataset.getRowKey(series).toString();
        	}	
        });

        barRenderer3D.setBaseItemLabelsVisible(true);
        barRenderer3D.setPositiveItemLabelPosition(new ItemLabelPosition(ItemLabelAnchor.INSIDE6, TextAnchor.BOTTOM_CENTER));
        barRenderer3D.setPositiveItemLabelPositionFallback(new ItemLabelPosition(ItemLabelAnchor.OUTSIDE12, TextAnchor.BOTTOM_CENTER));	       	        
        barRenderer3D.setPaint(Color.ORANGE);

        chartPositionDistribution.setChart(orderDistributionChart);
    }
    

    /**
     * <p>Renders a combined chart for query statistics, including query latency, through-put 
     * and cluster member consumption.</p>
     */
    public void renderQueryStatisticsChart() {
    	
    	//---------------------------------------------
    	//the aggregation time (latency) plot
    	LineAndShapeRenderer aggregationTimeRenderer = new LineAndShapeRenderer();
    	aggregationTimeRenderer.setShapesVisible(true);
    	aggregationTimeRenderer.setDrawOutlines(true);
    	aggregationTimeRenderer.setUseFillPaint(true);
    	aggregationTimeRenderer.setFillPaint(Color.WHITE);
    	aggregationTimeRenderer.setSeriesStroke(0, new BasicStroke(1.5f));
    	aggregationTimeRenderer.setSeriesOutlineStroke(0, new BasicStroke(1.0f));
    	aggregationTimeRenderer.setSeriesShape(0, new Ellipse2D.Double(-2.5, -2.0, 5.0, 5.0));
    	aggregationTimeRenderer.setSeriesPaint(0, Color.RED);
    		
    	NumberAxis aggregationTimeAxis = new NumberAxis("Query Time (ms)");
    	
    	CategoryPlot aggregationTimePlot = new CategoryPlot(aggregationTimeDataset, null, aggregationTimeAxis, aggregationTimeRenderer);
            	
    	//---------------------------------------------
    	//the aggregation through-put plot
    	LineAndShapeRenderer aggregationThroughputRenderer = new LineAndShapeRenderer();
    	aggregationThroughputRenderer.setShapesVisible(true);
    	aggregationThroughputRenderer.setDrawOutlines(true);
    	aggregationThroughputRenderer.setUseFillPaint(true);
    	aggregationThroughputRenderer.setFillPaint(Color.WHITE);
    	aggregationThroughputRenderer.setSeriesStroke(0, new BasicStroke(1.5f));
    	aggregationThroughputRenderer.setSeriesOutlineStroke(0, new BasicStroke(1.0f));
    	aggregationThroughputRenderer.setSeriesShape(0, new Ellipse2D.Double(-2.5, -2.0, 5.0, 5.0));
    	aggregationThroughputRenderer.setSeriesPaint(0, Color.BLACK);
   	
    	NumberAxis aggregationThroughputAxis = new NumberAxis("Query Rate (positions/sec)");
    	
    	CategoryPlot aggregationThroughputPlot = new CategoryPlot(aggregationThroughputDataset, null, aggregationThroughputAxis, aggregationThroughputRenderer);
    	
    	//---------------------------------------------
    	//the cluster member memory utilization plot
    	LineAndShapeRenderer memoryUtilizationRenderer = new LineAndShapeRenderer();
    	memoryUtilizationRenderer.setDrawOutlines(true);
    	memoryUtilizationRenderer.setUseFillPaint(true);
    	memoryUtilizationRenderer.setFillPaint(Color.WHITE);
    	memoryUtilizationRenderer.setPaint(Color.GREEN);
    	memoryUtilizationRenderer.setStroke(new BasicStroke(1.5f));

    	NumberAxis memoryUtilizationAxis = new NumberAxis("Memory Utilization (mb)");
    	
    	CategoryPlot memoryUtilizationPlot = new CategoryPlot(memoryConsumptionDataset, null, memoryUtilizationAxis, memoryUtilizationRenderer);

    	//---------------------------------------------
    	//create appropriate annotations for membership changes
        int queryNr = nrTasksExecuted - aggregationTimes.size() + 1;
        for (Tuple4<Integer, Integer, Long, Boolean> latency : aggregationTimes) {
        	String query = String.format("%d", queryNr++);
        	if (latency.getD() && queryNr-1 != 1) {
                CategoryMarker marker = new CategoryMarker(query, new Color(0, 0, 255, 25), new BasicStroke(1.0f));
                marker.setDrawAsLine(false);
                marker.setAlpha(1.0f);
                aggregationTimePlot.addDomainMarker(marker, Layer.BACKGROUND);
                aggregationThroughputPlot.addDomainMarker(marker, Layer.BACKGROUND);
                memoryUtilizationPlot.addDomainMarker(marker, Layer.BACKGROUND);
        	}
        } 
    	
    	
    	//---------------------------------------------
    	//construct the combined chart
        CombinedDomainCategoryPlot combinedPlot = new CombinedDomainCategoryPlot(new CategoryAxis("Sample"));
        combinedPlot.setGap(30.0);
        combinedPlot.add(aggregationTimePlot, 1);
        combinedPlot.add(aggregationThroughputPlot, 1);
        combinedPlot.add(memoryUtilizationPlot, 1);
        combinedPlot.setOrientation(PlotOrientation.VERTICAL);
 
        JFreeChart queryStatisticsChart = new JFreeChart("Performance Statistics", 
        												JFreeChart.DEFAULT_TITLE_FONT, 
        												combinedPlot, 
        												false);
        
        chartQueryStatistics.setChart(queryStatisticsChart);
    }
        
    
    /** 
     * This method is called from within the constructor to
     * initialize the form.
     */
    private void initComponents() {
        lblOracle = new javax.swing.JLabel();
        lblApplicationName = new javax.swing.JLabel();
        lblCopyright = new javax.swing.JLabel();
        chartPositionPopularity = new ChartPanel(null, true);
        chartQueryStatistics = new ChartPanel(null, true);
        chartPositionDistribution = new ChartPanel(null, true);

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
        lblOracle.setPreferredSize(new java.awt.Dimension(248, 33));
        lblOracle.setIcon(new javax.swing.ImageIcon(getClass().getResource("/com/oracle/coherence/swingdemo/oracle.gif")));
        
        lblApplicationName.setFont(new java.awt.Font("Lucida Grande", 1, 12));
        lblApplicationName.setText("Coherence Demonstration");

        lblCopyright.setFont(new java.awt.Font("Lucida Grande", 1, 10));
        lblCopyright.setText("Copyright 2007. Oracle Corporation. Version " + VERSION);

        chartPositionPopularity.setDoubleBuffered(true);
        chartPositionPopularity.setFocusable(false);
        chartPositionPopularity.setMaximumSize(null);
        chartPositionPopularity.setMinimumSize(null);
        chartPositionPopularity.setOpaque(true);
        chartPositionPopularity.setPreferredSize(null);
        chartPositionPopularity.setRequestFocusEnabled(false);
        chartPositionPopularity.setVerifyInputWhenFocusTarget(false);

        chartQueryStatistics.setDoubleBuffered(true);
        chartQueryStatistics.setFocusable(false);
        chartQueryStatistics.setMaximumSize(null);
        chartQueryStatistics.setMinimumSize(null);
        chartQueryStatistics.setOpaque(true);
        chartQueryStatistics.setPreferredSize(null);
        chartQueryStatistics.setRequestFocusEnabled(false);
        chartQueryStatistics.setVerifyInputWhenFocusTarget(false);

        chartPositionDistribution.setDoubleBuffered(true);
        chartPositionDistribution.setFocusable(false);
        chartPositionDistribution.setMaximumSize(null);
        chartPositionDistribution.setMinimumSize(null);
        chartPositionDistribution.setOpaque(true);
        chartPositionDistribution.setPreferredSize(null);
        chartPositionDistribution.setRequestFocusEnabled(false);
        chartPositionDistribution.setVerifyInputWhenFocusTarget(false);

        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(layout.createSequentialGroup()
                        .add(lblOracle, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(lblCopyright)
                            .add(lblApplicationName, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 431, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)))
                    .add(layout.createSequentialGroup()
                        .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(chartPositionPopularity, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 350, Short.MAX_VALUE)
                            .add(chartPositionDistribution, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 350, Short.MAX_VALUE))
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(chartQueryStatistics, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 350, Short.MAX_VALUE)))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(lblApplicationName)
                    .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING)
                        .add(lblCopyright)
                        .add(lblOracle, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(layout.createSequentialGroup()
                        .add(chartPositionPopularity, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 255, Short.MAX_VALUE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(chartPositionDistribution, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 255, Short.MAX_VALUE))
                    .add(chartQueryStatistics, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 518, Short.MAX_VALUE))
                .addContainerGap())
        );
        pack();

        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                //gracefully shutdown the scheduler
                scheduler.shutdown();

                //gracefully leave the cluster
                CacheFactory.shutdown();
           }
        });
        
      chartPositionPopularity.addComponentListener(new java.awt.event.ComponentAdapter() {
	      public void componentResized(java.awt.event.ComponentEvent evt) {
	    	  renderPositionPopularityChart();
	      }
	  });        

      chartPositionDistribution.addComponentListener(new java.awt.event.ComponentAdapter() {
	      public void componentResized(java.awt.event.ComponentEvent evt) {
	    	  renderPositionDistributionChart();
	      }
	  });        
    }
  
    
    /**
     * @param args the command line arguments
     */
    public static void main(String args[]) {
        java.awt.EventQueue.invokeLater(new Runnable() {
            public void run() {
                new GUI().setVisible(true);
            }
        });
    }    

    
    private javax.swing.JLabel lblOracle;
    private javax.swing.JLabel lblApplicationName;
    private javax.swing.JLabel lblCopyright;
    private ChartPanel chartPositionPopularity;
    private ChartPanel chartPositionDistribution;
    private ChartPanel chartQueryStatistics;
}
