package com.oracle.coherence.swingdemo.model;

import java.io.DataInput;
import java.io.DataOutput;
import java.io.IOException;

import com.tangosol.io.ExternalizableLite;
import com.tangosol.util.ExternalizableHelper;

/**
 * <p>An immutable class to represent a single position in a financial market for an equity (stock).</p>
 * 
 * @author Brian Oliver (brian.oliver@oracle.com)
 */
public class Position implements ExternalizableLite {

	private static final long serialVersionUID = -2557078539268609864L;
	
	
	/**
	 * <p>The unique identity of the {@link Position}.</p>
	 */
	private int id;
	
	
	/**
	 * <p>The symbol (ticker code) of the equity for the {@link Position}</p>
	 */
	private String symbol;
	
	
	/**
	 * <p>The number of shares for the {@link Position}.</p>
	 */
	private int amount;
	
	
	/**
	 * <p>The price at which the shares in the {@link Position} were acquired.</p>
	 */
	private double price;

	
	/**
	 * <p>Default Constructor (required and used only by {@link ExternalizableLite}).</p>
	 */
	public Position() {
		//required for Serializable and ExternalizableLite
	}
	
	
	/**
	 * <p>The standard constructor for a {@link Position}.</p>
	 * 
	 * @param id The unique identity of the {@link Position}
	 * @param symbol The symbol (ticker code) of the {@link Position}
	 * @param amount The number of shares (quantity) for the {@link Position}
	 * @param price The price at which the shares where acquired for the {@link Position}
	 */
	public Position(int id, String symbol, int amount, double price) {
		super();
		this.id = id;
		this.symbol = symbol;
		this.amount = amount;
		this.price = price;
	}
	
	
	/**
	 * <p>The unique identity of the {@link Position}.</p>
	 * 
	 * @return
	 */
	public int getId() {
		return id;
	}
	

	/**
	 * <p>The symbol (ticker code) of the equity (stock) for the {@link Position}.</p>
	 * 
	 * @return
	 */
	public String getSymbol() {
		return symbol;
	}


	/**
	 * <p>Determine the value at which the shares were acquired for the {@link Position}.</p>
	 * 
	 * @return
	 */
	public double getPrice() {
		return price;
	}
	
	
	/**
	 * <p>Determine the number of shares acquired for the {@link Position}.</p>
	 * 
	 * @return
	 */
	public int getAmount() {
		return amount;
	}


	/**
	 * <p>Determine the value of the {@link Position}. (value = price * quantity)</p>
	 * 
	 * @return
	 */
	public double getValue() {
		return getAmount() * getPrice();
	}
	
	
	/**
	 * {@inheritDoc}
	 */
	public void readExternal(DataInput dataInput) throws IOException {
		id = ExternalizableHelper.readInt(dataInput);
		symbol = ExternalizableHelper.readSafeUTF(dataInput);
		amount = ExternalizableHelper.readInt(dataInput);
		price = dataInput.readDouble();
	}
	
	
	/**
	 * {@inheritDoc}
	 */
	public void writeExternal(DataOutput dataOutput) throws IOException {
		ExternalizableHelper.writeInt(dataOutput, id);
		ExternalizableHelper.writeSafeUTF(dataOutput, symbol);
		ExternalizableHelper.writeInt(dataOutput, amount);
		dataOutput.writeDouble(price);
	}
}
